package xyz.zhouxy.plusone.system.util;

import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.annotation.Nonnull;

import xyz.zhouxy.plusone.exception.SysException;
import xyz.zhouxy.plusone.util.RandomUtil;

/**
 * 密码工具类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public final class PasswordUtil {
    private static final char[] SALT_BASE_CHAR_ARRAY = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ~`!@#$%^&*()_-+={}[]|\\:;\"',.<>?/"
            .toCharArray();

    /**
     * 将密码和随机盐混合，并进行哈希加密。
     *
     * @param password 密文密码
     * @param salt     随机盐
     * @return 哈希加密的结果
     */
    @Nonnull
    public static String hashPassword(@Nonnull String password, @Nonnull String salt) {
        int length = salt.length();
        int i = length > 0 ? length / 2 : 0;
        var passwordWithSalt = salt.substring(0, i)
                + password
                + salt.substring(i);
        try {
            return sha512Hex(passwordWithSalt);
        } catch (NoSuchAlgorithmException e) {
            throw new SysException(e);
        }
    }

    /**
     * 生成 24 位的字符串
     *
     * @return 生成的随机盐
     */
    public static String generateRandomSalt() {
        return RandomUtil.randomStr(SALT_BASE_CHAR_ARRAY, 24);
    }

    private PasswordUtil() {
        // 不允许实例化
        throw new IllegalStateException("Utility class");
    }

    @Nonnull
    @SuppressWarnings("null")
    private static String sha512Hex(String data) throws NoSuchAlgorithmException {
        MessageDigest messageDigest = MessageDigest.getInstance("SHA-512");
        messageDigest.update(data.getBytes(StandardCharsets.UTF_8));
        byte[] result = messageDigest.digest();
        return new BigInteger(1, result).toString(16);
    }
}
